package com.example.bomocloud

import com.example.bomocloud.auth.AuthService
import com.example.bomocloud.auth.OauthService
import com.example.bomocloud.home.HomeService
import com.example.bomocloud.theme.ThemeService
import com.example.bomocloud.video.VideoService
import okhttp3.OkHttpClient
import retrofit2.Retrofit
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory
import java.util.concurrent.CopyOnWriteArrayList

class BoMoCloud(private val clientId: String, private val clientSecret: String) {

    companion object {
        const val DEBUG_HOST = "http://preparing.bomostory.com/"
        const val RELEASE_HOST = "https://www.bomostory.com/"

        const val CLIENT_ID = "client_id"
        const val CLIENT_SECRET = "client_secret"

//        @Volatile
//        private var INSTANCE: BoMoCloud? = null //
//        private fun build(): BoMoCloud {
//            return BoMoCloud()
//        }
//
//        fun getInstance(): BoMoCloud =
//                INSTANCE ?: synchronized(this) {
//                    INSTANCE ?: build().also { INSTANCE = it }
//                }
    }

    val host = if (BuildConfig.DEBUG) DEBUG_HOST else RELEASE_HOST

    val authService: AuthService = buildAuthService()
    val oAuthService: OauthService = buildOAuthService()
    val homeService: HomeService = buildHomeService()
    val themeService: ThemeService = buildThemeService()
    val videoService: VideoService = buildVideoService()

    var refreshToken: String? = null

    val listener = CopyOnWriteArrayList<Listener>()

    private fun buildAuthService(): AuthService =
            getRetrofit(host + AuthService.BASE_URL).create(AuthService::class.java)

    private fun buildOAuthService(): OauthService =
            getRetrofit(host + OauthService.BASE_URL).create(OauthService::class.java)

    private fun buildHomeService(): HomeService =
            getRetrofit(host + HomeService.BASE_URL).create(HomeService::class.java)

    private fun buildThemeService(): ThemeService =
            getRetrofit(host + ThemeService.BASE_URL).create(ThemeService::class.java)

    private fun buildVideoService(): VideoService =
            getRetrofit(host + VideoService.BASE_URL).create(VideoService::class.java)

    private fun getRetrofit(baseUrl: String): Retrofit =
            Retrofit.Builder()
                    .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
                    .addConverterFactory(GsonConverterFactory.create())
                    .client(provideOkHttpClient())
                    .baseUrl(baseUrl)
                    .build()

    private fun provideOkHttpClient(): OkHttpClient =
            OkHttpClient.Builder()
                    .addInterceptor {
                        var request = it.request()
                        val httpUrl = request.url().newBuilder()
                                .addQueryParameter(CLIENT_ID, clientId)
                                .addQueryParameter(CLIENT_SECRET, clientSecret)
                                .build()
                        request = request.newBuilder().url(httpUrl).build()
                        it.proceed(request)
                    }
                    .authenticator(TokenAuthenticator(this))
                    .build()

    open class Listener {
        open fun onTokenRefreshed(accessToken: String, refreshToken: String) {}
    }
}
